@version 2.8
@warnings
@script generic
@name "Apply Point Info Node"

i;
msg = "";
meshes;
meshndxmax = 1;
ndxmodes = @"Object Clones", "Node Copies"@;
meshNDX, pointNDX, pointINCR, NDXmode, itemID;
ctl_meshndx, ctl_pointndx, ctl_pointincr, ctl_ndxmode;



generic
{
    if( hostVersion() < 9.0 ) {
        error( "Script requires LightWave version 9.0 or later.");
    }

    tempdir = getdir(TEMPDIR);
    selected = getSelected();
	meshes = getMeshes();
	test = getMeshes();
	first = getFirst( selected[1] );
    ids = getIds( selected );


	if( options() ) { 	
		//File management
	    orgFilename = Scene().filename;

	    tempdir = getdir( TEMPDIR );
	    readFilename = tempdir+"\\read.lws";
	    writeFilename = tempdir+"\\write.lws";

	    SaveSceneCopy( readFilename );
	    SaveSceneCopy( writeFilename );

	    readFile = File( readFilename, "r" );
	    writeFile = File( writeFilename, "w" );

	

		matchLine = getMatchLine( first.genus );
		if( readFile.isOpen() && writeFile.isOpen() ) {
			
			while( !readFile.eof() ) {
				line = readFile.read();
				writeFile.writeln( line );
				if( line == matchLine ) { //find the right item
					id = extractID(line, first.genus);
					index = matchID( id, ids );
					if( index ) {
						while( line != regexp("^PathAlignReliableDist")  ) { //find the right spot to insert text
							line = readFile.read();
							writeFile.writeln( line );
						}
						line = ApplyNode( meshNDX, (pointNDX++)*pointINCR, NDXmode, ids[index] );
						writeFile.writeln( line );
					}
				}
			}
		}


		//File management
		loadscene( writeFilename, orgFilename );
		
		readFile.close();
		writeFile.close();
		filedelete( readFilename );
		filedelete( writeFilename );

		
		Select( selected );
	}
}


options
{
    reqbegin("Apply Node Item Motion: Point Info v1.0");
    reqsize( 338, 110 );
    
    
    //CONTROLS
    ctl_meshndx = ctlpopup("Object", 1, meshes);
    ctl_pointndx = ctlinteger( "Point Index", 1 );
    ctl_pointincr = ctlinteger( "+", 1 );
    ctl_ndxmode = ctlchoice( "Indexation Mode", 0, ndxmodes, false );

    col = 5;
    row = 5;
    width = 326;
    height = 20;
    offset = width/2;
    ctlposition(ctl_meshndx, col, row, width, height, offset);
    row +=23;
    ctlposition(ctl_pointndx, col, row, width/4*3 - 8, height, offset);
    ctlposition(ctl_pointincr, width/4*3+2, row, width/4+4, height );
    row +=23;
    ctlposition(ctl_ndxmode, col, row, width, height, offset);

    return 0 if !reqpost();
    
    meshNDX = getvalue( ctl_meshndx );
    pointNDX = getvalue( ctl_pointndx );
    pointINCR = getvalue( ctl_pointincr );
    NDXmode = getvalue( ctl_ndxmode ) - 1;
        
    reqend();

    return 1;
}


/*
Applies the Motion Node motion modifier and adds a Point Info to an item.
var meshNDX the index of the mesh the point belongs to.
var pointID the ID of a point in the mesh.
var itemID the items own id (provided via the Motion Node).
*/
ApplyNode: meshNDX, pointNDX, NDXmode, itemID
{
    str = "";
    
    strArray = @ 
        "Plugin ItemMotionHandler 1 nodeItemMotion\n",
        "{ Nodes\n",
        "  { Root\n",
        "    Location 225 75\n",
        "    Zoom 1\n",
        "    Disabled 1\n",
        "  }\n",
        "  Version 1\n",
        "  { Nodes\n",
        "    Server \"Motion\"\n",
        "    { Tag\n",
        "      RealName \"Motion\"\n",
        "      Name \"Motion\"\n",
        "      Coordinates 0 0\n",
        "      Mode 1\n",
        "      { Data\n",
        "      }\n",
        "      Preview \"\"\n",
        "      Comment \"\"\n",
        "    }\n",
        "    Server \"DP_PointInfo\"\n",
        "    { Tag\n",
        "      RealName \"Point Info\"\n",
        "      Name \"Point Info (1)\"\n",
        "      Coordinates 150 0\n",
        "      Mode 1\n",
        "      { Data\n",
        "        "+meshNDX+"\n",
        "        "+pointNDX+"\n",
        "        "+NDXmode+"\n",
        "      }\n",
        "      Preview \"oPos\"\n",
        "      Comment \"\"\n",
        "    }\n",
        "  }\n",
        "  { Connections\n",
        "    NodeName \"Motion\"\n",
        "    InputName \"Position\"\n",
        "    InputNodeName \"Point Info (1)\"\n",
        "    InputOutputName \"wPos\"\n",
        "  }\n",
        "}\n",
        "AfterIK 0\n",
        "ItemID "+itemID+"\n", //non-hex itemid
        "EndPlugin"
    @;

    for( i=1; i<=sizeof(strArray); i++ ) {
        str += strArray[i];
    }

    return str;
}



getSelected {

   return Scene().getSelect();
}


getIds: items {
    foreach( item, items ) {
        ids += item.id;
    }
    
    return ids;
}


getMeshes {

    mesh = Mesh();
    while( mesh ) {
        meshes += mesh.name; //NAMES!
        mesh = mesh.next();
    }
    
    return meshes;
}

getFirst: item {

    itemType = item.genus;

    switch( itemType ) {
        case MESH:
            first = Mesh();
            break;
        
        case BONE:
            while( item.genus != MESH ) {
                item = item.parent;
            }
            first = Mesh(item.id).bone();
            break;
        
        case LIGHT:
            first = Light();
            break;
            
        case CAMERA:
            first = Camera();
            break;

        default:
            first = 0;
    }

    return first;
}


getMatchLine: genus {
	
	switch( genus ) {

		case MESH: 
		expression = regexp("^AddNullObject\|^LoadObjectLayer");
		break;
		
        case BONE:
		expression = regexp("^AddBone");
		break;
		
        case LIGHT:
		expression = regexp("^AddLight");
		break;
		
        case CAMERA:
		expression = regexp("^AddCamera");
		break;
		
		default:
		expression = nil;
	}
	
	return expression;
}

/*
Tests the line if it matches a prototype. If it does then it returns an id extracted from that line.
*/
extractID: line, genus {

	parts = parse( " ", line );

	switch( genus ) {

		case MESH: 
			if( line == regexp("^LoadObjectLayer") ) {
				string = "0x"+parts[3];
			}
			else { 
				string = "0x"+parts[2];
			}
			break;
		
        case BONE:
			string = "0x"+parts[2];
			break;
		
        case LIGHT:
			string = "0x"+parts[2];
			break;
		
        case CAMERA:
			string = "0x"+parts[2];
			break;
		
		default:
			string = "0x"+"00000000";
	}
	
	return integer(string);
}

matchID: id, ids {

	for( i=1; i<=sizeof(ids); i++ ) {
		if( id == ids[i] ) {
			return i;
		}
	}
	
	return 0;
}



getStringFromFile: file {

    str = "";

    if( file.isOpen() ) {
        while( !read.eof() ) {
            str += file.read();
        }        
    }

    return str;
}



Select: items
{
    SelectItem( items[items.size()].id );

    for( i = items.size() - 1; i > 0; i-- )
        AddToSelection( items[i].id );
}
















